SuperStrict

rem
bbdoc: Math for two-dimensional vectors
end rem

Module satyrias.Vector2

ModuleInfo "Name: Vector2"
ModuleInfo "Description: Vector2 is a type, that let's you work easily with two-dimensional vectors"
ModuleInfo "License: Public Domain"
ModuleInfo "Author: Satyrias"

Import brl.max2d

rem
bbdoc: Two-dimensional vector type
end rem

Type Vector2
	Field x:Float, y:Float
	
	Global VectorNull:Vector2 = Vector2.Create(0, 0)
	Global VectorLeft:Vector2 = Vector2.Create(-1, 0)
	Global VectorRight:Vector2 = Vector2.Create(1, 0)
	Global VectorUp:Vector2 = Vector2.Create(0, -1)
	Global VectorDown:Vector2 = Vector2.Create(0, 1)
	
	rem
	bbdoc: Creates a two-dimensional vector
	returns: Object of type 'Vector2'
	about:
	The function returns a two-dimensional vector as 'Vector2'
	end rem
	Function Create:Vector2(x:Float, y:Float)
		Local tmp:Vector2 = New Vector2
		
		tmp.x = x
		tmp.y = y
		
		Return tmp
	End Function
	
	rem
	bbdoc: Vector addition
	returns: Object of type 'Vector2'
	about:
	The function adds the vectors a & b and returns a two-dimensional vector as 'Vector2'
	end rem
	Function Addition:Vector2(a:Vector2, b:Vector2)
		Return Vector2.Create(a.x + b.x, a.y + b.y)
	End Function
	
	
	rem
	bbdoc: Vector subtraction
	returns: Object of type 'Vector2'
	about:
	The function subtracts the vector b from a and returns a two-dimensional vector as 'Vector2'
	end rem
	Function Subtraction:Vector2(a:Vector2, b:Vector2)
		Return Vector2.Create(a.x - b.x, a.y - b.y)
	End Function
	
	
	rem
	bbdoc: Vector multiplication
	returns: Object of type 'Vector2'
	about:
	The function multiplies the vector a by s and returns a two-dimensional vector as 'Vector2'
	end rem
	Function Multiplication:Vector2(a:Vector2, s:Float)
		Return Vector2.Create(a.x * s, a.y * s)
	End Function
	
	
	rem
	bbdoc: Vector division
	returns: Object of type 'Vector2'
	about:
	The function divides the vector a by s and returns a two-dimensional vector as 'Vector2'
	end rem
	Function Division:Vector2(a:Vector2, s:Float)
		If s <> 0
			Return Vector2.Create(a.x / s, a.y / s)
		End If
	End Function
	
	rem
	bbdoc: Dot product of two vectors
	returns: Scalar as 'Float'
	about:
	The function calculates the dot product of the vectors a & b and returns a scalar as 'Float'
	end rem
	Function DotProduct:Float(a:Vector2, b:Vector2)
		Return a.x * b.x + a.y * b.y
	End Function
	
	rem
	bbdoc: Angle between two vectors
	returns: Angle in degrees as 'Float'
	about:
	The function calculates the angle between the vectors a & b and returns it as 'Float'
	end rem
	Function AngleBetween:Float(a:Vector2, b:Vector2)
		Return ACos(Vector2.DotProduct(a, b) / (a.Magnitude() * b.Magnitude()))
	End Function
	
	rem
	bbdoc: Normal vector
	returns: Object of type 'Vector2'
	about:
	The function calculates one of the two (specified by s=0 or s=1) normal vectors and sets it's length to 1, then returns a two-dimensional vector as 'Vector2'
	end rem
	Function Normal:Vector2(a:Vector2, s:Byte = 0)
		If s
			Return Vector2.Division(Vector2.Create(a.y, -a.x), a.Magnitude())
		Else
			Return Vector2.Division(Vector2.Create(-a.y, a.x), a.Magnitude())
		End If
	End Function
	
	rem
	bbdoc: Vector's length
	returns: Scalar as 'Float'
	about:
	The method returns the magnitude of the vector as 'Float'
	end rem
	Method Magnitude:Float()
		Return Sqr(Self.x ^ 2 + Self.y ^ 2)
	End Method
	
	rem
	bbdoc: Unify Vector
	returns: Unfied/Normalized vector as 'Vector2'
	about:
	This function normalizes any vector with |a| != 0. The result is a vector a with |a| = 1
	end rem
	Method Normalize()
		If Self.Magnitude() <> 0
			Self.Divide(Self.Magnitude())
		End If
	End Method
	
	rem
	bbdoc: Inverted vector
	returns: Object of type 'Vector2'
	about:
	The method returns the inverted vector (multiply by -1) as two-dimensional vector as 'Vector2'
	end rem
	Method Invert:Vector2()
		Return Vector2.Multiplication(Self, -1)
	End Method
	
	rem
	bbdoc: Add a vector
	about:
	The method adds a to the vector
	end rem
	Method Add(a:Vector2)
		Local tmp:Vector2 = Vector2.Addition(Self, a)
		Self.x = tmp.x
		Self.y = tmp.y
	End Method
	
	rem
	bbdoc: Subtracts a vector
	about:
	The method subtracts a from the vector
	end rem
	Method Subtract(a:Vector2)
		Local tmp:Vector2 = Vector2.Subtraction(Self, a)
		Self.x = tmp.x
		Self.y = tmp.y
	End Method
	
	rem
	bbdoc: Multiply with a scalar
	about:
	The method multiplies the vector by s
	end rem
	Method Multiply(s:Float)
		Local tmp:Vector2 = Vector2.Multiplication(Self, s)
		Self.x = tmp.x
		Self.y = tmp.y
	End Method
	
	rem
	bbdoc: Divide by a scalar
	about:
	The method divides the vector by s
	end rem
	Method Divide(s:Float)
		Local tmp:Vector2
		
		If s <> 0
			tmp = Vector2.Division(Self, s)
			Self.x = tmp.x
			Self.y = tmp.y
		End If
	End Method
	
	rem
	bbdoc: Draw a representant
	about:
	Draws a representant of the vector at the coordinates given by repX & repY.
	The dot specifies the beginning of the vector, it's size is defined by dotSize (optional)
	end rem
	Method Draw(repX:Int, repY:Int, dotSize:Int = 4)
		DrawOval repX - dotSize/2, repY - dotSize/2, dotSize, dotSize
		DrawLine repX, repY, repX + Self.x, repY + Self.y
	End Method
End Type