SuperStrict

rem
bbdoc: Math for three-dimensional vectors
end rem

Module satyrias.Vector3

ModuleInfo "Name: Vector3"
ModuleInfo "Description: Vector3 is a type, that let's you work easily with three-dimensional vectors"
ModuleInfo "License: Public Domain"
ModuleInfo "Author: Satyrias"

Import brl.max2d

Type Vector3
	Global offXfZ:Float = -0.5, offYfZ:Float = 0.5
	
	Global VectorNull:Vector3 = Vector3.Create(0, 0, 0)
	Global VectorLeft:Vector3 = Vector3.Create(-1, 0, 0)
	Global VectorRight:Vector3 = Vector3.Create(1, 0, 0)
	Global VectorUp:Vector3 = Vector3.Create(0, -1, 0)
	Global VectorDown:Vector3 = Vector3.Create(0, 1, 0)
	Global VectorIn:Vector3 = Vector3.Create(0, 0, -1)
	Global VectorOut:Vector3 = Vector3.Create(0, 0, 1)
	
	Field x:Float, y:Float, z:Float
	
	rem
	bbdoc: Creates a three-dimensional vector
	returns: Object of type 'Vector3'
	about:
	The function returns a three-dimensional vector as 'Vector3'
	end rem
	Function Create:Vector3(x:Float, y:Float, z:Float)
		Local tmp:Vector3 = New Vector3
		
		tmp.x = x
		tmp.y = y
		tmp.z = z
		
		Return tmp
	End Function
	
	rem
	bbdoc: Vector addition
	returns: Object of type 'Vector3'
	about:
	The function adds the vectors a & b and returns a three-dimensional vector as 'Vector3'
	end rem
	Function Addition:Vector3(a:Vector3, b:Vector3)
		Return Vector3.Create(a.x + b.x, a.y + b.y, a.z + b.z)
	End Function
	
	
	rem
	bbdoc: Vector subtraction
	returns: Object of type 'Vector3'
	about:
	The function subtracts the vector b from a and returns a three-dimensional vector as 'Vector3'
	end rem
	Function Subtraction:Vector3(a:Vector3, b:Vector3)
		Return Vector3.Create(a.x - b.x, a.y - b.y, a.z - b.z)
	End Function
	
	rem
	bbdoc: Vector multiplication
	returns: Object of type 'Vector3'
	about:
	The function multiplies the vector a by s and returns a three-dimensional vector as 'Vector3'
	end rem
	Function Multiplication:Vector3(a:Vector3, s:Float)
		Return Vector3.Create(a.x * s, a.y * s, a.z * s)
	End Function
	
	
	rem
	bbdoc: Vector division
	returns: Object of type 'Vector3'
	about:
	The function divides the vector a by s and returns a three-dimensional vector as 'Vector3'
	end rem
	Function Division:Vector3(a:Vector3, s:Float)
		If s <> 0
			Return Vector3.Create(a.x / s, a.y / s, a.z / s)
		End If
	End Function
	
	rem
	bbdoc: Dot product of two vectors
	returns: Scalar as 'Float'
	about:
	The function calculates the dot product of the vectors a & b and returns a scalar as 'Float'
	end rem
	Function DotProduct:Float(a:Vector3, b:Vector3)
		Return a.x * b.x + a.y * b.y + a.z * b.z
	End Function
	
	rem
	bbdoc: Vector product of two vectors
	returns: Object of type 'Vector3'
	about:
	The function calculates the vector product of the vectors a & b and returns a three-dimensional vector as 'Vector3'
	end rem
	Function VectorProduct:Vector3(a:Vector3, b:Vector3)
		Return Vector3.Create(a.y * b.z - a.z - b.y, a.z * b.x - a.x * b.z, a.x * b.y - a.y * b.x)
	End Function
	
	rem
	bbdoc: Angle between two vectors
	returns: Angle in degrees as 'Float'
	about:
	The function calculates the angle between the vectors a & b and returns it as 'Float'
	end rem
	Function AngleBetween:Float(a:Vector3, b:Vector3)
		Return ACos(Vector3.DotProduct(a, b) / (a.Magnitude() * b.Magnitude()))
	End Function
	
	rem
	bbdoc: Normal vector
	returns: Object of type 'Vector3'
	about:
	The function calculates the normal vector in a right-handed system and sets it's length to 1, then returns a three-dimensional vector as 'Vector3'
	end rem
	Function Normal:Vector3(a:Vector3, b:Vector3)
		Local tmp:Vector3 = Vector3.VectorProduct(a, b)
		tmp.Normalize()
		Return tmp
	End Function
	
	rem
	bbdoc: Vector's length
	returns: Scalar as 'Float'
	about:
	The method returns the magnitude of the vector as 'Float'
	end rem
	Method Magnitude:Float()
		Return Sqr(Self.x ^ 2 + Self.y ^ 2 + Self.z ^ 2)
	End Method
	
	rem
	bbdoc: Unify Vector
	returns: Unfied/Normalized vector as 'Vector3'
	about:
	This function normalizes any vector with |a| != 0. The result is a vector a with |a| = 1
	end rem
	Method Normalize()
		If Self.Magnitude() <> 0
			Self.Divide(Self.Magnitude())
		End If
	End Method
	
	rem
	bbdoc: Inverted vector
	about:
	The method returns the inverted vector (multiply by -1) as two-dimensional vector as 'Vector3'
	end rem
	Method Invert:Vector3()
		Self.Multiply(-1)
	End Method
	
	rem
	bbdoc: Add a vector
	about:
	The method adds a to the vector
	end rem
	Method Add(a:Vector3)
		Local tmp:Vector3 = Vector3.Addition(Self, a)
		Self.x = tmp.x
		Self.y = tmp.y
		Self.z = tmp.z
	End Method
	
	rem
	bbdoc: Subtracts a vector
	about:
	The method subtracts a from the vector
	end rem
	Method Subtract(a:Vector3)
		Local tmp:Vector3 = Vector3.Subtraction(Self, a)
		Self.x = tmp.x
		Self.y = tmp.y
		Self.z = tmp.z
	End Method
	
	rem
	bbdoc: Multiply with a scalar
	about:
	The method multiplies the vector by s
	end rem
	Method Multiply(s:Float)
		Local tmp:Vector3 = Vector3.Multiplication(Self, s)
		Self.x = tmp.x
		Self.y = tmp.y
		Self.z = tmp.z
	End Method
	
	rem
	bbdoc: Divide by a scalar
	about:
	The method divides the vector by s
	end rem
	Method Divide(s:Float)
		Local tmp:Vector3
		
		If s <> 0
			tmp = Vector3.Division(Self, s)
			Self.x = tmp.x
			Self.y = tmp.y
			Self.z = tmp.z
		End If
	End Method
	
	rem
	bbdoc: Draw a representant
	about:
	Draws a representant of the vector at the coordinates given by repX & repY.
	The dot specifies the beginning of the vector, it's size is defined by dotSize (optional)
	end rem
	Method Draw(repX:Int, repY:Int, dotSize:Int = 4)
		DrawOval repX - dotSize / 2, repY - dotSize / 2, dotSize, dotSize
		DrawLine repX, repY, repX + Self.x + offXfZ * Self.z, repY - Self.y + offYfZ*Self.z
	End Method
End Type